@@ -1,5 +1,4 @@
-__version__ = "0.12.9"
-
+from spb._version import __version__
 from spb.functions import (
     plot,
     plot_parametric,
@@ -8,12 +7,12 @@ from spb.functions import (
     plot3d_parametric_line,
     plot3d_parametric_surface,
     plot_implicit,
-    polar_plot,
-    geometry_plot,
+    plot_polar,
+    plot_geometry,
 )
 from spb.plot_data import get_plot_data, smart_plot
-from spb.vectors import vector_plot
-from spb.ccomplex.complex import complex_plot
+from spb.vectors import plot_vector
+from spb.ccomplex.complex import plot_complex, plot3d_complex
 
 # from spb.interactive import iplot
 # from spb.backends.plotgrid import plotgrid
@@ -25,6 +24,8 @@ p3d = plot3d
 p3dpl = plot3d_parametric_line
 p3dps = plot3d_parametric_surface
 implicit_plot = plot_implicit
-plot_polar = polar_plot
-plot_geometry = geometry_plot
-plot_complex = complex_plot
+polar_plot = plot_polar
+geometry_plot = plot_geometry
+complex_plot = plot_complex
+complex_plot3d = plot3d_complex
+vector_plot = plot_vector
