@@ -1,13 +1,15 @@
 from spb.defaults import cfg
 from spb.backends.base_backend import Plot
 from spb.backends.utils import convert_colormap
-from spb.utils import get_vertices_indices, get_seeds_points
+from spb.utils import get_vertices_indices
+from spb.backends.utils import compute_streamtubes
 import k3d
 import numpy as np
 import warnings
 from matplotlib.tri import Triangulation
 from mergedeep import merge
 import colorcet as cc
+import os
 
 # TODO:
 # 1. load the plot with menu minimized
@@ -46,6 +48,11 @@ class K3DBackend(Plot):
         If True, apply a color map to the meshes/surface. If False, solid
         colors will be used instead. Default to True.
 
+
+    See also
+    ========
+
+    Plot, MatplotlibBackend, PlotlyBackend, BokehBackend
     """
 
     _library = "k3d"
@@ -70,7 +77,8 @@ class K3DBackend(Plot):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         if self._get_mode() != 0:
-            raise ValueError("Sorry, K3D backend only works within Jupyter Notebook")
+            raise ValueError(
+                "Sorry, K3D backend only works within Jupyter Notebook")
 
         self._bounds = []
         self._clipping = []
@@ -198,138 +206,22 @@ class K3DBackend(Plot):
 
                 self._fig += surf
 
-            elif s.is_3Dvector and self._kwargs.get("streamlines", False):
+            elif s.is_3Dvector and s.is_streamlines:
                 xx, yy, zz, uu, vv, ww = s.get_data()
                 # K3D doesn't like masked arrays, so filled them with NaN
                 xx, yy, zz, uu, vv, ww = [
-                    np.ma.filled(t) if isinstance(t, np.ma.core.MaskedArray) else t
-                    for t in [xx, yy, zz, uu, vv, ww]
-                ]
-                magnitude = np.sqrt(uu ** 2 + vv ** 2 + ww ** 2)
-                min_mag = min(magnitude.flatten())
-                max_mag = max(magnitude.flatten())
+                    np.ma.filled(t) if isinstance(t, np.ma.core.MaskedArray)
+                    else t for t in [xx, yy, zz, uu, vv, ww]]
 
-                import vtk
-                from vtk.util import numpy_support
-
-                vector_field = np.array([uu.flatten(), vv.flatten(), ww.flatten()]).T
-                vtk_vector_field = numpy_support.numpy_to_vtk(
-                    num_array=vector_field, deep=True, array_type=vtk.VTK_FLOAT
-                )
-                vtk_vector_field.SetName("vector_field")
-
-                points = vtk.vtkPoints()
-                points.SetNumberOfPoints(s.n2 * s.n1 * s.n3)
-                for i, (x, y, z) in enumerate(
-                    zip(xx.flatten(), yy.flatten(), zz.flatten())
-                ):
-                    points.SetPoint(i, [x, y, z])
-
-                grid = vtk.vtkStructuredGrid()
-                grid.SetDimensions([s.n2, s.n1, s.n3])
-                grid.SetPoints(points)
-                grid.GetPointData().SetVectors(vtk_vector_field)
+                vertices, magn = compute_streamtubes(
+                    xx, yy, zz, uu, vv, ww, self._kwargs)
 
                 stream_kw = self._kwargs.get("stream_kw", dict())
-                starts = stream_kw.pop("starts", None)
-                max_prop = stream_kw.pop("max_prop", 500)
-
-                streamer = vtk.vtkStreamTracer()
-                streamer.SetInputData(grid)
-                streamer.SetMaximumPropagation(max_prop)
-
-                if starts is None:
-                    seeds_points = get_seeds_points(xx, yy, zz, uu, vv, ww)
-                    seeds = vtk.vtkPolyData()
-                    points = vtk.vtkPoints()
-                    for p in seeds_points:
-                        points.InsertNextPoint(p)
-                    seeds.SetPoints(points)
-                    streamer.SetSourceData(seeds)
-                    streamer.SetIntegrationDirectionToForward()
-                elif isinstance(starts, dict):
-                    if not all([t in starts.keys() for t in ["x", "y", "z"]]):
-                        raise KeyError(
-                            "``starts`` must contains the following keys: "
-                            + "'x', 'y', 'z', whose values are going to be "
-                            + "lists of coordinates."
-                        )
-                    seeds_points = np.array([starts["x"], starts["y"], starts["z"]]).T
-                    seeds = vtk.vtkPolyData()
-                    points = vtk.vtkPoints()
-                    for p in seeds_points:
-                        points.InsertNextPoint(p)
-                    seeds.SetPoints(points)
-                    streamer.SetSourceData(seeds)
-                    streamer.SetIntegrationDirectionToBoth()
-                else:
-                    npoints = stream_kw.get("npoints", 200)
-                    radius = stream_kw.get("radius", None)
-                    center = 0, 0, 0
-                    if not radius:
-                        xmin, xmax = min(xx[0, :, 0]), max(xx[0, :, 0])
-                        ymin, ymax = min(yy[:, 0, 0]), max(yy[:, 0, 0])
-                        zmin, zmax = min(zz[0, 0, :]), max(zz[0, 0, :])
-                        radius = max(
-                            [abs(xmax - xmin), abs(ymax - ymin), abs(zmax - zmin)]
-                        )
-                        center = (xmax - xmin) / 2, (ymax - ymin) / 2, (zmax - zmin) / 2
-                    seeds = vtk.vtkPointSource()
-                    seeds.SetRadius(radius)
-                    seeds.SetCenter(*center)
-                    seeds.SetNumberOfPoints(npoints)
-
-                    streamer.SetSourceConnection(seeds.GetOutputPort())
-                    streamer.SetIntegrationDirectionToBoth()
-
-                streamer.SetComputeVorticity(0)
-                streamer.SetIntegrator(vtk.vtkRungeKutta4())
-                streamer.Update()
-
-                streamline = streamer.GetOutput()
-                streamlines_points = numpy_support.vtk_to_numpy(
-                    streamline.GetPoints().GetData()
-                )
-                streamlines_velocity = numpy_support.vtk_to_numpy(
-                    streamline.GetPointData().GetArray("vector_field")
-                )
-                streamlines_speed = np.linalg.norm(streamlines_velocity, axis=1)
-
-                vtkLines = streamline.GetLines()
-                vtkLines.InitTraversal()
-                point_list = vtk.vtkIdList()
-
-                lines = []
-                lines_attributes = []
-
-                while vtkLines.GetNextCell(point_list):
-                    start_id = point_list.GetId(0)
-                    end_id = point_list.GetId(point_list.GetNumberOfIds() - 1)
-                    l = []
-                    v = []
-
-                    for i in range(start_id, end_id):
-                        l.append(streamlines_points[i])
-                        v.append(streamlines_speed[i])
-
-                    lines.append(np.array(l))
-                    lines_attributes.append(np.array(v))
-
-                count = sum([len(l) for l in lines])
-                vertices = np.nan * np.zeros((count + (len(lines) - 1), 3))
-                attributes = np.zeros(count + (len(lines) - 1))
-                c = 0
-                for k, (l, a) in enumerate(zip(lines, lines_attributes)):
-                    vertices[c : c + len(l), :] = l
-                    attributes[c : c + len(l)] = a
-                    if k < len(lines) - 1:
-                        c = c + len(l) + 1
-
-                skw = dict(width=0.1, shader="mesh", compression_level=9)
+                skw = dict(width=0.1, shader="mesh")
                 if self._use_cm and ("color" not in stream_kw.keys()):
                     skw["color_map"] = next(self._cm)
-                    skw["color_range"] = [min_mag, max_mag]
-                    skw["attribute"] = attributes
+                    skw["color_range"] = [np.nanmin(magn), np.nanmax(magn)]
+                    skw["attribute"] = magn
                 else:
                     col = stream_kw.pop("color", next(self._cl))
                     if not isinstance(col, int):
@@ -337,8 +229,7 @@ class K3DBackend(Plot):
                     stream_kw["color"] = col
 
                 self._fig += k3d.line(
-                    vertices.astype(np.float32), **merge({}, skw, stream_kw)
-                )
+                    vertices.astype(np.float32), **merge({}, skw, stream_kw))
 
             elif s.is_3Dvector:
                 xx, yy, zz, uu, vv, ww = s.get_data()
@@ -476,7 +367,7 @@ class K3DBackend(Plot):
                     self._fig.objects[i].color_range = [z.min(), z.max()]
 
                 elif s.is_vector and s.is_3D:
-                    if self._kwargs.get("streamlines", False):
+                    if s.is_streamlines:
                         raise NotImplementedError
 
                     xx, yy, zz, uu, vv, ww = self.series[i].get_data()
@@ -489,7 +380,6 @@ class K3DBackend(Plot):
                         colors = k3d.helpers.map_colors(magnitude, colormap, [])
                         vec_colors = np.zeros(2 * len(colors))
                         for j, c in enumerate(colors):
-                            # print("j", j, "c", c)
                             vec_colors[2 * j] = c
                             vec_colors[2 * j + 1] = c
                         vec_colors = vec_colors.astype(np.uint32)
@@ -512,6 +402,8 @@ class K3DBackend(Plot):
         # self._fig.auto_rendering = True
 
     def show(self):
+        """Visualize the plot on the screen."""
+
         if len(self._fig.objects) != len(self.series):
             self._process_series(self._series)
         self.plot_shown = True
@@ -534,20 +426,54 @@ class K3DBackend(Plot):
             ]
 
     def save(self, path, **kwargs):
+        """ Export the plot to a static picture or to an interactive html file.
+
+        Notes
+        =====
+
+        K3D-Jupyter is only capable of exporting:
+
+        1. '.png' pictures: refer to [#fn1]_ to visualize the available keyword
+           arguments.
+        2. '.html' files: when exporting a fully portable html file, by default
+           the required Javascript libraries will be loaded with a CDN. Set
+           ``include_js=True`` to include all the javascript code in the html
+           file: this will create a bigger file size, but can be run without
+           internet connection.
+
+        References
+        ==========
+        .. [#fn1] https://k3d-jupyter.org/k3d.html#k3d.plot.Plot.fetch_screenshot
+
+        """
         if not self.plot_shown:
             raise ValueError(
                 "K3D-Jupyter requires the plot to be shown on the screen "
                 + "before saving it."
             )
 
-        @self._fig.yield_screenshots
-        def _func():
-            self._fig.fetch_screenshot()
-            screenshot = yield
-            with open(path, "wb") as f:
-                f.write(screenshot)
+        ext = os.path.splitext(path)[1]
+        if not ext:
+            path += ".png"
 
-        _func()
+        if ext in [".html", "htm"]:
+            with open(path, 'w') as f:
+                include_js = kwargs.pop("include_js", False)
+                self.fig.snapshot_include_js = include_js
+                f.write(self.fig.get_snapshot(**kwargs))
+        elif ext == ".png":
+            @self._fig.yield_screenshots
+            def _func():
+                self._fig.fetch_screenshot(**kwargs)
+                screenshot = yield
+                with open(path, "wb") as f:
+                    f.write(screenshot)
+
+            _func()
+        else:
+            raise ValueError(
+                "K3D-Jupyter can only export '.png' images or " +
+                "html files.")
 
 
 KB = K3DBackend
