@@ -1,6 +1,6 @@
 from spb.defaults import cfg
 from spb.backends.base_backend import Plot
-from spb.utils import get_seeds_points
+from spb.backends.utils import get_seeds_points
 import plotly.graph_objects as go
 from plotly.figure_factory import create_quiver, create_streamline
 from mergedeep import merge
@@ -28,69 +28,48 @@ class PlotlyBackend(Plot):
     ==========
 
     aspect : str, optional
-        Default to ``"auto"``. Possible values:
-        
+        Set the aspect ratio of the plot. Default to ``"auto"``.
+        Possible values:
+
         - ``"equal"``: sets equal spacing on the axis of a 2D plot.
         - ``"cube"``, ``"auto"`` for 3D plots.
 
     contour_kw : dict, optional
         A dictionary of keywords/values which is passed to Plotly's contour
         function to customize the appearance.
-        Refer to the following web pages to learn more about customization:
-        https://plotly.com/python/contour-plots/
-        https://plotly.com/python/builtin-colorscales/
+        Refer to [#fn1]_ and [#fn2]_  to learn more about customization.
 
     line_kw : dict, optional
         A dictionary of keywords/values which is passed to Plotly's scatter
         functions to customize the appearance.
-        Refer to the following web pages to learn more about customization:
-        https://plotly.com/python/line-and-scatter/
-        https://plotly.com/python/3d-scatter-plots/
+        Refer to [#fn3]_ and [#fn4]_ to learn more about customization.
 
     quiver_kw : dict, optional
         A dictionary of keywords/values which is passed to Plotly's quivers
         function to customize the appearance.
 
         - For 2D vector fields, default to: ``dict( scale = 0.075 )``
-          Refer to this documentation page:
-          https://plotly.com/python/quiver-plots/
+          Refer to [#fn5]_ for more options.
         - For 3D vector fields, default to: ``dict( sizemode = "absolute", sizeref = 40 )``
-          Refer to this documentation page:
-          https://plotly.com/python/cone-plot/
+          Refer [#fn6]_ for more options.
 
     surface_kw : dict, optional
         A dictionary of keywords/values which is passed to Plotly's
         Surface function to customize the appearance.
-        Refer to this documentation page:
-        https://plotly.com/python/3d-surface-plots/
+        Refer to [#fn7]_ for more options.
 
     stream_kw : dict, optional
         A dictionary of keywords/values which is passed to Plotly's
         streamlines function to customize the appearance.
-        
+
         - For 2D vector fields, defaul to: ``dict( arrow_scale = 0.15 )``
-          Refer to this documentation page:
-          https://plotly.com/python/streamline-plots/
-        - For 3D vector fields, default to:
-
-          .. code-block::
-
-                dict(
-                        sizeref = 0.3,
-                        starts = dict(
-                            x = seeds_points[:, 0],
-                            y = seeds_points[:, 1],
-                            z = seeds_points[:, 2]
-                    ))
-
-          where ``seeds_points`` are the starting points of the streamlines.
-          Refer to this documentation page:
-          https://plotly.com/python/streamtube-plot/
+          Refer to [#fn8]_ for more options.
+        - For 3D vector fields, default to: ``dict( sizeref = 0.3 )``
+          Refer to [#fn9]_ for more options.
 
     theme : str, optional
         Set the theme. Default to ``"plotly_dark"``. Find more Plotly themes at
-        the following page:
-        https://plotly.com/python/templates/
+        [#fn10]_ .
 
     use_cm : boolean, optional
         If True, apply a color map to the meshes/surface. If False, solid
@@ -100,15 +79,25 @@ class PlotlyBackend(Plot):
         Visualize the wireframe lines on surfaces. Default to False.
         Note that it definitely have a negative impact on the performances, and
         it does not work with ``iplot``.
+    
+    References
+    ==========
+    .. [#fn1] https://plotly.com/python/contour-plots/
+    .. [#fn2] https://plotly.com/python/builtin-colorscales/
+    .. [#fn3] https://plotly.com/python/line-and-scatter/
+    .. [#fn4] https://plotly.com/python/3d-scatter-plots/
+    .. [#fn5] https://plotly.com/python/quiver-plots/
+    .. [#fn6] https://plotly.com/python/cone-plot/
+    .. [#fn7] https://plotly.com/python/3d-surface-plots/
+    .. [#fn8] https://plotly.com/python/streamline-plots/
+    .. [#fn9] https://plotly.com/python/streamtube-plot/
+    .. [#fn10] https://plotly.com/python/templates/
 
 
-    Notes
-    =====
-
-    In order to export the plots, the user also need to install the packages
-    listed in the following page:
-    https://plotly.com/python/static-image-export/
+    See also
+    ========
 
+    Plot, MatplotlibBackend, BokehBackend, K3DBackend
     """
 
     _library = "plotly"
@@ -490,8 +479,7 @@ class PlotlyBackend(Plot):
             elif s.is_vector:
                 if s.is_2Dvector:
                     xx, yy, uu, vv = s.get_data()
-                    streamlines = self._kwargs.get("streamlines", False)
-                    if streamlines:
+                    if s.is_streamlines:
                         # NOTE: currently, it is not possible to create streamlines with
                         # a color scale: https://community.plotly.com/t/how-to-make-python-quiver-with-colorscale/41028
                         # default values
@@ -517,14 +505,20 @@ class PlotlyBackend(Plot):
                         self._fig.add_trace(quiver.data[0])
                 else:
                     xx, yy, zz, uu, vv, ww = s.get_data()
-                    streamlines = self._kwargs.get("streamlines", False)
-                    if streamlines:
-                        seeds_points = get_seeds_points(xx, yy, zz, uu, vv, ww)
+                    if s.is_streamlines:
+                        stream_kw = self._kwargs.get("stream_kw", dict()).copy()
+                        seeds_points = get_seeds_points(
+                            xx, yy, zz, uu, vv, ww, to_numpy=True, **stream_kw)
 
                         # default values
                         skw = dict(
-                            colorscale=next(self._cm),
+                            colorscale=(
+                                next(self._cm)
+                                if self._use_cm
+                                else self._solid_colorscale()
+                            ),
                             sizeref=0.3,
+                            showscale=self.legend and self._use_cm,
                             colorbar=self._create_colorbar(ii, s.label),
                             starts=dict(
                                 x=seeds_points[:, 0],
@@ -532,8 +526,14 @@ class PlotlyBackend(Plot):
                                 z=seeds_points[:, 2],
                             ),
                         )
-                        # user-provided values
-                        stream_kw = self._kwargs.get("stream_kw", dict())
+
+                        # the following keywords must be removed otherwise
+                        # Plotly's Stramtube raises error
+                        _kws = ["starts", "max_prop", "npoints", "radius"]
+                        for _k in _kws:
+                            if _k in stream_kw.keys():
+                                stream_kw.pop(_k)
+
                         self._fig.add_trace(
                             go.Streamtube(
                                 x=xx.flatten(),
@@ -604,14 +604,14 @@ class PlotlyBackend(Plot):
                                     colorscale=[
                                         "rgb(%s, %s, %s)" % tuple(c) for c in colors
                                     ],
-                                    color=[-self.pi, self.pi],
+                                    color=[-np.pi, np.pi],
                                     colorbar=dict(
                                         tickvals=[
-                                            -self.pi,
-                                            -self.pi / 2,
+                                            -np.pi,
+                                            -np.pi / 2,
                                             0,
-                                            self.pi / 2,
-                                            self.pi,
+                                            np.pi / 2,
+                                            np.pi,
                                         ],
                                         ticktext=[
                                             "-&#x3C0;",
@@ -659,11 +659,11 @@ class PlotlyBackend(Plot):
                             title="Argument",
                             titleside="right",
                             tickvals = [
-                                -self.pi,
-                                -self.pi / 2,
+                                -np.pi,
+                                -np.pi / 2,
                                 0,
-                                self.pi / 2,
-                                self.pi,
+                                np.pi / 2,
+                                np.pi,
                             ],
                             ticktext = [
                                 "-&#x3C0;",
@@ -673,8 +673,8 @@ class PlotlyBackend(Plot):
                                 "&#x3C0;",
                             ]
                         ),
-                        cmin = -self.pi,
-                        cmax = self.pi,
+                        cmin = -np.pi,
+                        cmax = np.pi,
                         colorscale=colorscale,
                         # colorscale=colormap if self._use_cm else colorscale,
                         surfacecolor=angle,
@@ -753,8 +753,7 @@ class PlotlyBackend(Plot):
                         self.fig.data[i]["z"] = ones
 
                 elif s.is_vector and s.is_3D:
-                    streamlines = self._kwargs.get("streamlines", False)
-                    if streamlines:
+                    if s.is_streamlines:
                         raise NotImplementedError
                     _, _, _, u, v, w = self.series[i].get_data()
                     self.fig.data[i]["u"] = u.flatten()
@@ -763,8 +762,7 @@ class PlotlyBackend(Plot):
 
                 elif s.is_vector:
                     x, y, u, v = self.series[i].get_data()
-                    streamlines = self._kwargs.get("streamlines", False)
-                    if streamlines:
+                    if s.is_streamlines:
                         streams = create_streamline(x[0, :], y[:, 0], u, v)
                         data = streams.data[0]
                         # TODO: iplot doesn't work with 2D streamlines. Why?
@@ -802,12 +800,12 @@ class PlotlyBackend(Plot):
                         m, M = min(angle.flatten()), max(angle.flatten())
                         # show pi symbols on the colorbar if the range is close
                         # enough to [-pi, pi]
-                        if (abs(m + self.pi) < 1e-02) and (abs(M - self.pi) < 1e-02):
+                        if (abs(m + np.pi) < 1e-02) and (abs(M - np.pi) < 1e-02):
                             self.fig.data[i]["colorbar"]["tickvals"] = [
                                 m,
-                                -self.pi / 2,
+                                -np.pi / 2,
                                 0,
-                                self.pi / 2,
+                                np.pi / 2,
                                 M,
                             ]
                             self.fig.data[i]["colorbar"]["ticktext"] = [
@@ -884,19 +882,36 @@ class PlotlyBackend(Plot):
         )
 
     def show(self):
+        """Visualize the plot on the screen."""
         if len(self._fig.data) != len(self.series):
             self._process_series(self.series)
         self._fig.show()
 
     def save(self, path, **kwargs):
+        """ Export the plot to a static picture or to an interactive html file.
+
+        Refer to [#fn11]_ and [#fn12]_ to visualize all the available keyword
+        arguments.
+
+        Notes
+        =====
+        In order to export static pictures, the user also need to install the
+        packages listed in [#fn11]_.
+
+        References
+        ==========
+        .. [#fn11] https://plotly.com/python/static-image-export/
+        .. [#fn12] https://plotly.com/python/interactive-html-export/
+
+        """
         self._process_series(self._series)
         self._update_layout()
 
         ext = os.path.splitext(path)[1]
-        if (ext == ".html") or (ext == ".htm"):
+        if ext.lower() in [".html", ".html"]:
             self.fig.write_html(path, **kwargs)
         else:
-            self._fig.write_image(path)
+            self._fig.write_image(path, **kwargs)
 
 
 PB = PlotlyBackend
