@@ -20,19 +20,10 @@ if you care at all about performance.
 
 
 from sympy import (
-    Expr,
-    Tuple,
-    Symbol,
-    oo,
-    Piecewise,
-    piecewise_fold,
-    UniversalSet,
-    EmptySet,
-    FiniteSet,
-    Interval,
-    Union,
+    Expr, Tuple, Symbol, oo,
+    Piecewise, piecewise_fold,
+    UniversalSet, EmptySet, FiniteSet, Interval, Union,
 )
-
 from spb.backends.base_backend import Plot
 from spb.utils import _plot_sympify, _check_arguments, _unpack_args
 
@@ -41,114 +32,87 @@ from spb.utils import _plot_sympify, _check_arguments, _unpack_args
 # the same in the `SymPyDocTestFinder`` in `sympy/testing/runtests.py`
 
 from spb.series import (
-    LineOver1DRangeSeries,
-    Parametric2DLineSeries,
-    Parametric3DLineSeries,
-    SurfaceOver2DRangeSeries,
-    ParametricSurfaceSeries,
-    ContourSeries,
-    ImplicitSeries,
-    _set_discretization_points,
-    List2DSeries,
-    GeometrySeries,
+    LineOver1DRangeSeries, Parametric2DLineSeries, Parametric3DLineSeries,
+    SurfaceOver2DRangeSeries, ContourSeries, ParametricSurfaceSeries,
+    ImplicitSeries, _set_discretization_points,
+    List2DSeries, GeometrySeries
 )
 
 
-def _get_endpoints(i, _min, _max):
-    """Given the end points of a local range, compute the
-    appropriate end points of the interval in such a way that the interval is
-    contained in the local range.
-
-    Returns
-    =======
-        a, b : float
-            The appropriate end points
-        skip : boolean
-            Indicate whether to add the series or to skip it. Series whose
-            domain doesn't overlap with the provided local range [_min, _max]
-            will be discarded.
-    """
-    a, b, lopen, ropen = i.args
-
-    skip = False
-    if (a >= _max) or (b <= _min):
-        skip = True
-
-    if not skip:
-        if a < _min:
-            a = _min
-        if b > _max:
-            b = _max
-
-        eps = (b - a) / 1e06
-        if lopen:
-            a += eps
-        if ropen:
-            b -= eps
-    return a, b, skip
-
-
 def _process_piecewise(piecewise, _range, label, **kwargs):
-    s = EmptySet
+    """Extract the pieces of an univariate Piecewise function and create the
+    necessary series for an univariate plot.
+
+    Notes
+    =====
+
+    As a design choice, the following implementation reuses the existing
+    classes, instead of creating a new one to deal with Piecewise. Here, each
+    piece is going to create at least one series. If a piece is using a union
+    of coditions (for example, ``((x < 0) | (x > 2))``), than two or more series
+    of the same expression are created (for example, one covering ``x < 0`` and
+    the other covering ``x > 2``), both having the same label.
+
+    With this approach we can assign different color and label to each piece,
+    thus making it obvious where discontinuities are located.
+
+    However, if a piece is outside of the provided plotting range, then it will
+    not be added to the plot. This may lead to not-complete plots in some
+    backend, such as BokehBackend, which is capable of auto-recompute the data
+    on mouse drag. If the user drags the mouse over an area previously not
+    shown (thus no data series), there won't be any line on the plot in this
+    area.
+    """
     series = []
-    _min, _max = _range[1], _range[2]
+    irange = Interval(_range[1], _range[2], False, False)
 
-    # for the label, attach the number of the piece
-    count = 1
     if "Piecewise(" in label:
         # piecewise string representation are usually very long. Cut it short
         # if the user didn't specify any custom label.
         label = "P"
 
-    for arg in piecewise.args:
-        expr, cond = arg.args
-        cond = cond.as_set()
-        if isinstance(cond, Interval):
-            s = s.union(cond)
-            a, b, skip = _get_endpoints(cond, _min, _max)
-            if not skip:
-                series.append(
-                    LineOver1DRangeSeries(expr, (_range[0], a, b),
-                        label + str(count))
-                )
-                count += 1
-        elif isinstance(cond, FiniteSet):
-            s = s.union(cond)
+    def func(expr, _set, c, from_union=False):
+        if isinstance(_set, Interval):
+            series.append(
+                LineOver1DRangeSeries(
+                    expr, (_range[0], _set.args[0], _set.args[1]),
+                    label + str(c), **kwargs))
+            if not from_union:
+                c += 1
+        elif isinstance(_set, FiniteSet):
             loc, val = [], []
-            for _loc in cond.args:
+            for _loc in _set.args:
                 loc.append(float(_loc))
-                val.append(float(piecewise.evalf(subs={_range[0]: _loc})))
-            series.append(List2DSeries(loc, val, label + str(count),
+                val.append(float(expr.evalf(subs={_range[0]: _loc})))
+            series.append(List2DSeries(loc, val, label + str(c),
                     is_point=True))
-            count += 1
-        elif isinstance(cond, UniversalSet.func):
-            # at this point the condition should be UniversalSet (or True)
-            # meaning the complementary part of the domain.
-            # NOTE: there should not be any more arg after this one...
-            s = s.complement(Interval(_min, _max, True, True))
-            if isinstance(s, Union):
-                s1 = [t for t in s.args if isinstance(t, Interval)]
-                s2 = [t for t in s.args if isinstance(t, FiniteSet)]
-            else:
-                s1, s2 = [s], []
+            if not from_union:
+                c += 1
+        elif isinstance(_set, Union):
+            for _s in _set.args:
+                c = func(expr, _s, c, from_union=True)
+        elif isinstance(_set, EmptySet.func):
+            # in this case, some pieces are outside of the provided range.
+            # don't add any series, but increase the counter nonetheless so that
+            # there is one-to-one correspondance between the expression and
+            # what is plotted.
+            if not from_union:
+                c += 1
+        else:
+            raise TypeError(
+                "Unhandle situation:\n" +
+                "expr: {}\ncond: {}\ntype(cond): {}".format(str(expr),
+                    _set, type(_set)))
+
+        return c
+
+    piecewise = piecewise_fold(piecewise)
+    expr_cond = piecewise.as_expr_set_pairs()
+    # for the label, attach the number of the piece
+    count = 1
+    for expr, cond in expr_cond:
+        count = func(expr, irange.intersection(cond), count)
 
-            for t in s1:
-                a, b, skip = _get_endpoints(t, _min, _max)
-                if not skip:
-                    series.append(
-                        LineOver1DRangeSeries(
-                            expr, (_range[0], a, b), label + str(count)
-                        )
-                    )
-                    count += 1
-            for t in s2:
-                loc, val = [], []
-                for _loc in t.args:
-                    loc.append(float(_loc))
-                    val.append(float(expr.evalf(subs={_range[0]: _loc})))
-                series.append(List2DSeries(loc, val, label + str(count),
-                        is_point=True))
-                count += 1
     return series
 
 
@@ -157,10 +121,10 @@ def _build_line_series(*args, **kwargs):
     decompose it in such a way that each argument gets its own series.
     """
     series = []
+    pp = kwargs.get("process_piecewise", True)
     for arg in args:
         expr, r, label = arg
-        if expr.has(Piecewise):
-            expr = piecewise_fold(expr)
+        if expr.has(Piecewise) and pp:
             series += _process_piecewise(expr, r, label, **kwargs)
         else:
             series.append(LineOver1DRangeSeries(*arg, **kwargs))
@@ -194,7 +158,7 @@ def plot(*args, show=True, **kwargs):
         range: (symbol, min, max)
             A 3-tuple denoting the range of the x variable. Default values:
             `min=-10` and `max=10`.
-        
+
         label : str, optional
             The label to be shown in the legend. If not provided, the string
             representation of ``expr`` will be used.
@@ -244,6 +208,11 @@ def plot(*args, show=True, **kwargs):
         Default to False. If True, generate a polar plot of a curve with radius
         `expr` as a function of the range
 
+    process_piecewise : boolean
+        Default to True. If a ``Piecewise`` expression is encountered, extract
+        and plot the pieces separately. This ensure a correct representation
+        of discontinuities. If False, discontinuities won't be visible.
+
     show : bool, optional
         The default value is set to ``True``. Set show to ``False`` and
         the function will not display the plot. The returned instance of
@@ -607,7 +576,7 @@ def plot3d_parametric_line(*args, show=True, **kwargs):
     n : int
         The range is uniformly sampled at ``n`` number of points. Default value
         is 300.
-    
+
     show : bool, optional
         The default value is set to ``True``. Set show to ``False`` and
         the function will not display the plot. The returned instance of
@@ -628,7 +597,7 @@ def plot3d_parametric_line(*args, show=True, **kwargs):
 
     ylabel : str, optional
         Label for the y-axis.
-    
+
     zlabel : str, optional
         Label for the z-axis.
 
@@ -637,7 +606,7 @@ def plot3d_parametric_line(*args, show=True, **kwargs):
 
     ylim : (float, float), optional
         Denotes the y-axis limits, ``(min, max)``.
-    
+
     zlim : (float, float), optional
         Denotes the z-axis limits, ``(min, max)``.
 
@@ -736,7 +705,7 @@ def plot3d(*args, show=True, **kwargs):
         range_y: (symbol, min, max)
             A 3-tuple denoting the range of the y variable. Default values:
             `min=-10` and `max=10`.
-        
+
         label : str, optional
             The label to be shown in the legend.  If not provided, the string
             representation of ``expr`` will be used.
@@ -752,7 +721,7 @@ def plot3d(*args, show=True, **kwargs):
     n : int, optional
         The x and y ranges are sampled uniformly at ``n`` of points.
         It overrides ``n1`` and ``n2``.
-    
+
     show : bool, optional
         The default value is set to ``True``. Set show to ``False`` and
         the function will not display the plot. The returned instance of
@@ -773,7 +742,7 @@ def plot3d(*args, show=True, **kwargs):
 
     ylabel : str, optional
         Label for the y-axis.
-    
+
     zlabel : str, optional
         Label for the z-axis.
 
@@ -782,7 +751,7 @@ def plot3d(*args, show=True, **kwargs):
 
     ylim : (float, float), optional
         Denotes the y-axis limits, ``(min, max)``.
-    
+
     zlim : (float, float), optional
         Denotes the z-axis limits, ``(min, max)``.
 
@@ -899,7 +868,7 @@ def plot3d_parametric_surface(*args, show=True, **kwargs):
 
         range_v: (symbol, min, max)
             A 3-tuple denoting the range of the ``v`` variable.
-        
+
         label : str, optional
             The label to be shown in the legend.  If not provided, the string
             representation of the expression will be used.
@@ -915,7 +884,7 @@ def plot3d_parametric_surface(*args, show=True, **kwargs):
     n : int, optional
         The u and v ranges are sampled uniformly at ``n`` of points.
         It overrides ``n1`` and ``n2``.
-    
+
     show : bool, optional
         The default value is set to ``True``. Set show to ``False`` and
         the function will not display the plot. The returned instance of
@@ -936,7 +905,7 @@ def plot3d_parametric_surface(*args, show=True, **kwargs):
 
     ylabel : str, optional
         Label for the y-axis.
-    
+
     zlabel : str, optional
         Label for the z-axis.
 
@@ -945,7 +914,7 @@ def plot3d_parametric_surface(*args, show=True, **kwargs):
 
     ylim : (float, float), optional
         Denotes the y-axis limits, ``(min, max)``.
-    
+
     zlim : (float, float), optional
         Denotes the z-axis limits, ``(min, max)``.
 
@@ -1024,7 +993,7 @@ def plot_contour(*args, show=True, **kwargs):
        >>> from sympy import symbols, cos, exp
        >>> from spb.functions import plot_contour
        >>> x, y = symbols('x, y')
-    
+
     Contour of a function of two variables.
 
     .. plot::
@@ -1037,7 +1006,7 @@ def plot_contour(*args, show=True, **kwargs):
        Plot object containing:
        [0]: contour: exp(-x**2/10 - y**2/10)*cos(x**2 + y**2) for x over (-5.0, 5.0) and y over (-5.0, 5.0)
 
-    
+
     See Also
     ========
 
@@ -1266,7 +1235,7 @@ def plot_polar(*args, **kwargs):
         >>> from sympy import symbols, sin, pi
         >>> from spb.functions import plot_polar
         >>> x = symbols('x')
-       
+
 
     .. plot::
         :context: close-figs
@@ -1332,7 +1301,7 @@ def plot_geometry(*args, show=True, **kwargs):
 
     ylabel : str, optional
         Label for the y-axis.
-    
+
     zlabel : str, optional
         Label for the z-axis.
 
@@ -1341,10 +1310,10 @@ def plot_geometry(*args, show=True, **kwargs):
 
     ylim : (float, float), optional
         Denotes the y-axis limits, ``(min, max)``.
-    
+
     zlim : (float, float), optional
         Denotes the z-axis limits, ``(min, max)``.
-    
+
 
     Examples
     ========
@@ -1436,7 +1405,7 @@ def plot_geometry(*args, show=True, **kwargs):
        >>> plot_geometry(
        ...      (Point3D(5, 5, 5), "center"),
        ...      (Line3D(Point3D(-2, -3, -4), Point3D(2, 3, 4)), "line"),
-       ...      (Plane((0, 0, 0), (1, 1, 1)), 
+       ...      (Plane((0, 0, 0), (1, 1, 1)),
        ...          (x, -5, 5), (y, -4, 4), (z, -10, 10)))
        Plot object containing:
        [0]: geometry entity: Point3D(5, 5, 5)
