@@ -187,7 +187,6 @@ def _build_series(*args, **kwargs):
                 return r
 
             _cls, nexpr, npar = mapping[pt]
-            print("pt", pt, _cls, nexpr, npar)
             k = str(nexpr) + str(npar)
             if k == "00":
                 args = [exprs, ranges, label]
@@ -245,8 +244,8 @@ def get_plot_data(*args, **kwargs):
     ==========
 
     pt : str, optional
-        Specify which kind of data the user would like to obtain. Default 
-        value is ``None``, indicating the function will use automatic 
+        Specify which kind of data the user would like to obtain. Default
+        value is ``None``, indicating the function will use automatic
         detection. Possible values are:
 
         - ``"p"``: to specify a line plot.
@@ -262,72 +261,47 @@ def get_plot_data(*args, **kwargs):
         - ``"v2d"``: to specify a 2D vector plot.
         - ``"v3d"``: to specify a 3D vector plot.
 
+    get_series : boolean, optional
+        If False, it returns the numerical data associated to the provided
+        expression. If True, it returns the data series object which can be
+        used to generate the data. Default to False.
+
     Examples
     ========
 
-    .. plot::
-        :context: reset
-        :format: doctest
-        :include-source: True
-
-        >>> from sympy import (symbols, pi, sin, cos, exp, Plane,
-        ...     Matrix, gamma, I, sqrt, Abs)
-        >>> from spb.plot_data import get_plot_data
-        >>> u, v, x, y, z = symbols('u, v, x:z')
+    >>> from sympy import (symbols, pi, sin, cos, exp, Plane,
+    ...     Matrix, gamma, I, sqrt, Abs)
+    >>> from spb.plot_data import get_plot_data
+    >>> u, v, x, y, z = symbols('u, v, x:z')
 
     Data from a function with a single variable (2D line):
 
-    .. plot::
-        :context: close-figs
-        :format: doctest
-        :include-source: True
-
-        >>> xx, yy = get_plot_data(cos(x), (x, -5, 5))
+    >>> xx, yy = get_plot_data(cos(x), (x, -5, 5))
 
     Here, ``xx`` and ``yy`` are two lists of coordinates.
 
     Data from a function with two variables (surface):
 
-    .. plot::
-        :context: close-figs
-        :format: doctest
-        :include-source: True
-
-        >>> xx, yy, zz = get_plot_data(cos(x * y), (x, -5, 5), (y, -10, 10))
+    >>> xx, yy, zz = get_plot_data(cos(x * y), (x, -5, 5), (y, -10, 10))
 
     Here, ``xx, yy, zz`` are two-dimensional numpy arrays. `
     `xx, yy`` represent the mesh grid.
 
     Data from a 2D parametric function with one variable (2D line):
 
-    .. plot::
-        :context: close-figs
-        :format: doctest
-        :include-source: True
-
-        >>> xx, yy, param = get_plot_data(cos(u), sin(u), (u, 0, 2 * pi))
+    >>> xx, yy, param = get_plot_data(cos(u), sin(u), (u, 0, 2 * pi))
 
     Here, ``xx, yy`` are two lists of coordinates.
 
     Data from a 3D parametric function with one variables (3D line):
 
-    .. plot::
-        :context: close-figs
-        :format: doctest
-        :include-source: True
-
-        >>> xx, yy, zz, param = get_plot_data(cos(u), sin(u), u, (u, -5, 5))
+    >>> xx, yy, zz, param = get_plot_data(cos(u), sin(u), u, (u, -5, 5))
 
     Here, ``xx, yy, zz`` are three lists of coordinates.
 
     Data from an implicit relation:
 
-    .. plot::
-        :context: close-figs
-        :format: doctest
-        :include-source: True
-
-        >>> data = get_plot_data(x > y)
+    >>> data = get_plot_data(x > y)
 
     Here, ``data`` depends on the specific case. Its shape could be:
 
@@ -341,48 +315,38 @@ def get_plot_data(*args, **kwargs):
       This is returned by objects of type non-equalities (greater than,
       less than, ...).
 
+    Get data from a symbolic geometry entity:
+
+    >>> xx, yy = get_plot_data(Ellipse(Point(0, 0), 5, 1), n=10)
+
+    Get data from a plane: being a 3D entity, it requires three ranges, even
+    if the plane is parallel to one of the xy, yz, xz planes.
+
+    >>> xx, yy, zz = get_plot_data(Plane((0, 0, 0), (1, 1, 0)),
+    ...     (x, -10, 10), (y, -10, 10), (z, -10, 10))
+
     Get the necessary data to plot a 3D vector field over a slice plane:
 
-    .. plot::
-        :context: close-figs
-        :format: doctest
-        :include-source: True
-
-        >>> xx, yy, zz, uu, vv, ww = get_plot_data(
-        ...     Matrix([z, y, x]), (x, -5, 5), (y, -5, 5), (z, -5, 5),
-        ...     slice = Plane((-2, 0, 0), (1, 1, 1)), n=5)
+    >>> xx, yy, zz, uu, vv, ww = get_plot_data(
+    ...     Matrix([z, y, x]), (x, -5, 5), (y, -5, 5), (z, -5, 5),
+    ...     slice = Plane((-2, 0, 0), (1, 1, 1)), n=5)
 
     Here ``xx, yy, zz, uu, vv, ww`` are three dimensional numpy arrays.
 
     Get the real part of a complex function over a real range:
 
-    .. plot::
-        :context: close-figs
-        :format: doctest
-        :include-source: True
-
-        >>> xx, real = get_plot_data(sqrt(x), (x, -3, 3),
-        ...     real=True, imag=False)
+    >>> xx, real = get_plot_data(sqrt(x), (x, -3, 3),
+    ...     real=True, imag=False)
 
     Get the magnitude and argument of a complex function over a real range:
 
-    .. plot::
-        :context: close-figs
-        :format: doctest
-        :include-source: True
-
-        >>> expr = 1 + exp(-Abs(x)) * sin(I * sin(5 * x))
-        >>> xx, mag, arg = get_plot_data(expr, (x, -3, 3), absarg=True)
+    >>> expr = 1 + exp(-Abs(x)) * sin(I * sin(5 * x))
+    >>> xx, mag, arg = get_plot_data(expr, (x, -3, 3), absarg=True)
 
     Compute a complex function over a complex range:
 
-    .. plot::
-        :context: close-figs
-        :format: doctest
-        :include-source: True
-
-        >>> xx, yy, abs, arg, img, colorscale = get_plot_data(gamma(z), 
-        ...     (z, -3 - 3*I, 3 + 3*I))
+    >>> xx, yy, abs, arg, img, colorscale = get_plot_data(gamma(z),
+    ...     (z, -3 - 3*I, 3 + 3*I))
 
     Here, ``xx, yy`` are 2D arrays representing the real and the imaginary part
     of the domain, respectively. ``abs`` and ``arg`` are the absolute value and
@@ -397,7 +361,11 @@ def get_plot_data(*args, **kwargs):
     smart_plot
 
     """
-    return _build_series(*args, **kwargs).get_data()
+    get_series = kwargs.pop("get_series", False)
+    s = _build_series(*args, **kwargs)
+    if get_series:
+        return s
+    return s.get_data()
 
 
 def smart_plot(*args, show=True, **kwargs):
@@ -496,7 +464,7 @@ def smart_plot(*args, show=True, **kwargs):
         ...     slice=Plane((0, 0, 0), (1, 1, 1)),
         ...     quiver_kw={"length": 0.25})
         Plot object containing:
-        [0]: sliced 3D vector series: [z, y, x] over (x, -5.0, 5.0), (y, -5.0, 5.0), (z, -5.0, 5.0) with plane Plane(Point3D(0, 0, 0), (1, 1, 1))
+        [0]: sliced 3D vector series: [z, y, x] over (x, -5.0, 5.0), (y, -5.0, 5.0), (z, -5.0, 5.0) at Plane(Point3D(0, 0, 0), (1, 1, 1))
 
     See also
     ========
